<?php

namespace Drupal\rabbit_hole;

use Drupal\Core\Entity\ContentEntityInterface;

/**
 * Interface BehaviourSettingsManagerInterface.
 */
interface BehaviorSettingsManagerInterface {

  /**
   * Save behavior settings for an entity or bundle.
   *
   * @param array $settings
   *   The settings for the BehaviorSettings entity.
   * @param string $entity_type_id
   *   The entity type of bundle (e.g. node_type) as a string.
   * @param string|null $entity_id
   *   The bundle entity ID as a string.
   */
  public function saveBehaviorSettings(array $settings, string $entity_type_id, ?string $entity_id = NULL): void;

  /**
   * Get behavior settings for the given entity type and bundle.
   *
   * @param string $entity_type_id
   *   The entity type ID (e.g. node, user).
   * @param string $bundle
   *   The entity bundle name.
   */
  public function getBehaviorSettings(string $entity_type_id, string $bundle): array;

  /**
   * An entity's rabbit hole configuration, or the default if it does not exist.
   *
   * Return an entity's rabbit hole configuration or, failing that, the default
   * configuration for the bundle (which itself will call the base default
   * configuration if necessary).
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to apply rabbit hole behavior on.
   *
   * @return array
   *   An array of values from the entity's fields matching the base properties
   *   added by rabbit hole.
   */
  public function getEntityBehaviorSettings(ContentEntityInterface $entity): array;

  /**
   * Checks if an entity type is enabled in the Rabbit Hole settings.
   *
   * @param string $entity_type_id
   *   The entity type ID.
   *
   * @return bool
   *   TRUE if an entity type is enabled, FALSE otherwise.
   */
  public function entityTypeIsEnabled(string $entity_type_id): bool;

  /**
   * Enables Rabbit Hole support for an entity type.
   *
   * @param string $entity_type_id
   *   Entity type ID.
   */
  public function enableEntityType(string $entity_type_id): void;

  /**
   * Disables Rabbit Hole support for an entity type.
   *
   * @param string $entity_type_id
   *   Entity type ID.
   */
  public function disableEntityType(string $entity_type_id): void;

}
